#include <iostream>
#include "svg.h"
#include <vector>
#include <string>

void svg_begin(double width, double height)
{
    std::cout << "<?xml version='1.0' encoding='UTF-8'?>\n";
    std::cout << "<svg ";
    std::cout << "width='" << width << "' ";
    std::cout << "height='" << height << "' ";
    std::cout << "viewBox='0 0 " << width << " " << height << "' ";
    std::cout << "xmlns='http://www.w3.org/2000/svg'>\n";
}

void svg_end()
{
    std::cout << "</svg>\n";
}

void svg_text(double left, double baseline, std::string text)
{
    std::cout << "<text x='" << left << "' y='" << baseline << "'>" << text << "</text>";
}

void show_histogram_svg(const std::vector<size_t>& bins)
{
    const auto IMAGE_WIDTH = 400;
    const auto IMAGE_HEIGHT = 300;
    const auto TEXT_LEFT = 20;
    const auto TEXT_BASELINE = 20;
    const auto TEXT_WIDTH = 50;
    const auto BIN_HEIGHT = 30;
    const auto BLOCK_WIDTH = 10;
    const auto MAX_AVAIL_WIDTH = IMAGE_WIDTH - TEXT_WIDTH - 20; //    

    svg_begin(IMAGE_WIDTH, IMAGE_HEIGHT);

    //    bins
    size_t max_bin = 0;
    for (size_t bin : bins)
    {
        if (bin > max_bin) max_bin = bin;
    }

    if (max_bin * BLOCK_WIDTH > MAX_AVAIL_WIDTH)
    {
        //  
        double scale = 1.0;
        scale = static_cast<double>(MAX_AVAIL_WIDTH) / (max_bin * BLOCK_WIDTH);
        double top = 0;
        for (size_t bin : bins)
        {
            const double bin_width = BLOCK_WIDTH * bin * scale;

            svg_text(TEXT_LEFT, top + TEXT_BASELINE, std::to_string(bin));
            svg_rect(TEXT_WIDTH, top, bin_width, BIN_HEIGHT);
            top += BIN_HEIGHT;
        }
    }
    else
    {
        double top = 0;
        for (size_t bin : bins)
        {
            const double bin_width = BLOCK_WIDTH * bin;
            svg_text(TEXT_LEFT, top + TEXT_BASELINE, std::to_string(bin));
            svg_rect(TEXT_WIDTH, top, bin_width, BIN_HEIGHT);
            top += BIN_HEIGHT;
        }
    }

    svg_end();
}

void svg_rect(double x, double y, double width, double height, std::string stroke, std::string fill)
{
    std::cout << "<rect x='" << x << "' y='" << y << "' width='" << width << "' height='" << height << "' stroke='" << stroke << "' fill='" << fill << "' />";
}
